function [A,B,lam,fr,Crdb]=Make_OEM(FS,ord,crttype,ftype,iira,iirb)
%    [A,B]=Make_OEM(FS,ord,crttype,ftype,iira,iirb)
%
% This function can be used to design a fixed FIR or IIR filter
% for modeling the transfer function of outer and middle ear.
%
% FS is the sampling rate, ord is the order of a FIR approximation,
% crttype is a string which specifies the type of correction:
% 'ELC' - Equal Loudness Curves at 60 dB (default)
% 'MAP' - Minimum Audible Pressure at ear canal
% 'MAF' - Minimum Audible Field 
%
% Four different filter types (variable ftype) are supported:
% 'FIR' - A linear phase FIR filter (default)
% 'IIR' - A minimum phase IIR filter derived from the previous
% 'WFIR'- A warped FIR filter (from the Warping Toolbox)
% 'WIIR'- A warped IIR filter (from the Warping Toolbox)
%
% If an IIR filter is used, one must specify also the number of
% zeros (iira) and poles (iirb).
%
% This function uses (with permission) a modified version of 
% Toshio Irino's OutMidCrct-function from his Gammachirp package.

% This function is a part of HUTear - free Matlab toolbox for auditory
% modeling 2.0. The toolbox is available at 
% http://www.acoustics.hut.fi/software/HUTear.

ord0=ord;

if nargin<3 | isempty(crttype), crttype='ELC';end
if nargin<4, ftype='FIR';end

switch ftype,
  case 'FIR',
    [Cr,fr,Crdb]=OutMidCrct(crttype,ord0,FS);
    fr=fr/max(fr);
    A=fir2(ord0,fr,Cr); % Design a FIR
    B=1;
  case 'IIR',
    [Cr,fr,Crdb]=OutMidCrct(crttype,ord0,FS);
    fr=fr/max(fr);
    A=fir2(ord0,fr,Cr); % Design a FIR
    [xx,xxx]=rceps(A); % Minimum phase response
    [A,B] = prony(xxx, iira, iirb); % Design an IIR filter
  case 'WFIR',
    [Cr,fr,Crdb]=OutMidCrct2(crttype,ord0,FS);
    fr=fr/max(fr);
    A=fir2(ord0,fr,Cr); % Design a FIR on the warped domain
    B=1;
    lam=barkwarp(FS);
  case 'WIIR',
   [Cr,fr,Crdb]=OutMidCrct2(crttype,ord0,FS);
    fr=fr/max(fr);
    A=fir2(ord0,fr,Cr); % Design a warped FIR
    [xx,xxx]=rceps(A); % Minimum phase response
    [A,B] = prony(xxx, iira, iirb); % Design a warped IIR filter  
    lam=barkwarp(FS);
end



